@@ -2,17 +2,20 @@ import os
 import socket
 import json
 import logging
+import ssl
+import http.client
 from datetime import datetime
 from tempfile import NamedTemporaryFile
 from urllib import request
 
 from cinp.common import URI
 
-__CLIENT_VERSION__ = '0.9.8'
+__CLIENT_VERSION__ = '0.18.0'
 __CINP_VERSION__ = '0.9'
 
-__all__ = [ 'Timeout', 'ResponseError', 'InvalidRequest', 'InvalidSession',
-            'NotAuthorized', 'NotFound', 'ServerError', 'CInP' ]
+__all__ = [ 'Timeout', 'ResponseError', 'DetailedInvalidRequest',
+            'InvalidRequest', 'InvalidSession', 'NotAuthorized',
+            'NotFound', 'ServerError', 'CInP' ]
 
 
 class Timeout( Exception ):
@@ -27,6 +30,20 @@ class InvalidRequest( Exception ):
   pass
 
 
+class DetailedInvalidRequest( InvalidRequest ):
+  def __init__( self, data ):
+    for name in ( 'class', 'error', 'message' ):
+      setattr( self, name, None )  # this makes sure it is at least set
+      try:
+        setattr( self, name, data[ name ] )
+        del data[ name ]
+      except KeyError:
+        pass
+
+    self.data = data
+    super().__init__( self.message )
+
+
 class InvalidSession( Exception ):
   pass
 
@@ -43,13 +60,8 @@ class ServerError( Exception ):
   pass
 
 
-class HTTPErrorProcessorPassthrough( request.HTTPErrorProcessor ):
-  def http_response( self, request, response ):
-    return response
-
-
 class CInP():
-  def __init__( self, host, root_path, proxy=None ):
+  def __init__( self, host, root_path, proxy=None, verify_ssl=True ):
     super().__init__()
 
     if not host.startswith( ( 'http:', 'https:' ) ):
@@ -64,10 +76,21 @@ class CInP():
 
     self.uri = URI( root_path )
 
+    self.opener = request.OpenerDirector()
+
     if self.proxy:  # not doing 'is not None', so empty strings don't try and proxy   # have a proxy option to take it from the envrionment vars
-      self.opener = request.build_opener( HTTPErrorProcessorPassthrough, request.ProxyHandler( { 'http': self.proxy, 'https': self.proxy } ) )
+      self.opener.add_handler( request.ProxyHandler( { 'http': self.proxy, 'https': self.proxy } ) )
     else:
-      self.opener = request.build_opener( HTTPErrorProcessorPassthrough, request.ProxyHandler( {} ) )
+      self.opener.add_handler( request.ProxyHandler( {} ) )
+
+    self.opener.add_handler( request.HTTPHandler() )
+    if hasattr( http.client, 'HTTPSConnection' ):
+      if not verify_ssl:
+        self.opener.add_handler( request.HTTPSHandler( context=ssl._create_unverified_context() ) )
+      else:
+        self.opener.add_handler( request.HTTPSHandler() )
+
+    self.opener.add_handler( request.UnknownHandler() )
 
     self.opener.addheaders = [
                                 ( 'User-Agent', 'python CInP client {0}'.format( __CLIENT_VERSION__ ) ),
@@ -123,7 +146,10 @@ class CInP():
       header_map[ 'Content-Type' ] = 'application/octet-stream'
       if not hasattr( data, 'read' ):
         raise InvalidRequest( 'data must be an readable stream' )
-      verb = 'POST'  # not to be handled by CInP on the other end, but by a bolt on file upload handler
+      verb = 'POST'  # not to be handled by CInP on the other end, but by a file upload handler
+
+    elif verb == 'RAWGET':  # not a CINP verb, just using it to bypass some checking here in _request
+      verb = 'GET'
 
     else:
       header_map[ 'Content-Type' ] = 'application/json;charset=utf-8'
@@ -134,8 +160,7 @@ class CInP():
         data = json.dumps( data, default=JSONDefault ).encode( 'utf-8' )
 
     url = '{0}{1}'.format( self.host, uri )
-    req = request.Request( url, data=data, headers=header_map )
-    req.get_method = lambda: verb
+    req = request.Request( url, data=data, headers=header_map, method=verb )
     try:
       resp = self.opener.open( req, timeout=timeout )
 
@@ -158,7 +183,22 @@ class CInP():
     if http_code not in ( 200, 201, 202, 400, 401, 403, 404, 500 ):
       raise ResponseError( 'HTTP code "{0}" unhandled'.format( resp.code ) )
 
-    logging.debug( 'cinp: got http code "{0}"'.format( http_code ) )
+    logging.debug( 'cinp: got HTTP code "{0}"'.format( http_code ) )
+
+    if http_code == 401:
+      resp.close()
+      logging.warning( 'cinp: Invalid Session' )
+      raise InvalidSession()
+
+    if http_code == 403:
+      resp.close()
+      logging.warning( 'cinp: Not Authorized' )
+      raise NotAuthorized()
+
+    if http_code == 404:
+      resp.close()
+      logging.warning( 'cinp: Not Found' )
+      raise NotFound()
 
     buff = str( resp.read(), 'utf-8' ).strip()
     if not buff:
@@ -183,24 +223,13 @@ class CInP():
 
     if http_code == 400:
       try:
-        message = data[ 'message' ]
+        data[ 'message' ]
+        e = DetailedInvalidRequest( data )
       except ( KeyError, ValueError, TypeError ):
-        message = buff[ 0:200 ]
+        e = InvalidRequest( buff[ 0:200 ] )
 
-      logging.warning( 'cinp: Invalid Request "{0}"'.format( message ) )
-      raise InvalidRequest( message )
-
-    if http_code == 401:
-      logging.warning( 'cinp: Invalid Session' )
-      raise InvalidSession()
-
-    if http_code == 403:
-      logging.warning( 'cinp: Not Authorized' )
-      raise NotAuthorized()
-
-    if http_code == 404:
-      logging.warning( 'cinp: Not Found' )
-      raise NotFound()
+      logging.warning( 'cinp: Invalid Request "{0}"'.format( e.args[0] ) )
+      raise e
 
     if http_code == 500:
       if isinstance( data, dict ):
@@ -242,7 +271,7 @@ class CInP():
     ( http_code, data, _ ) = self._request( 'DESCRIBE', uri, timeout=timeout )
 
     if http_code != 200:
-      logging.warning( 'cinp: unexpected HTTP Code "{0}" for DESCRIBE'.format( http_code ) )
+      logging.warning( 'cinp: Unexpected HTTP Code "{0}" for DESCRIBE'.format( http_code ) )
       raise ResponseError( 'Unexpected HTTP Code "{0}" for DESCRIBE'.format( http_code ) )
 
     return data
@@ -268,7 +297,7 @@ class CInP():
     ( http_code, id_list, header_map ) = self._request( 'LIST', uri, data=filter_value_map, header_map=header_map, timeout=timeout )
 
     if http_code != 200:
-      logging.warning( 'cinp: unexpected HTTP Code "{0}" for LIST'.format( http_code ) )
+      logging.warning( 'cinp: Unexpected HTTP Code "{0}" for LIST'.format( http_code ) )
       raise ResponseError( 'Unexpected HTTP Code "{0}" for LIST'.format( http_code ) )
 
     if not isinstance( id_list, list ):
@@ -296,7 +325,7 @@ class CInP():
     ( http_code, rec_values, header_map ) = self._request( 'GET', uri, header_map=header_map, timeout=timeout )
 
     if http_code != 200:
-      logging.warning( 'cinp: unexpected HTTP Code "{0}" for GET'.format( http_code ) )
+      logging.warning( 'cinp: Unexpected HTTP Code "{0}" for GET'.format( http_code ) )
       raise ResponseError( 'Unexpected HTTP Code "{0}" for GET'.format( http_code ) )
 
     if not isinstance( rec_values, dict ):
@@ -316,7 +345,7 @@ class CInP():
     ( http_code, rec_values, header_map ) = self._request( 'CREATE', uri, data=values, timeout=timeout )
 
     if http_code != 201:
-      logging.warning( 'cinp: unexpected HTTP Code "{0}" for CREATE'.format( http_code ) )
+      logging.warning( 'cinp: Unexpected HTTP Code "{0}" for CREATE'.format( http_code ) )
       raise ResponseError( 'Unexpected HTTP Code "{0}" for CREATE'.format( http_code ) )
 
     if not isinstance( rec_values, dict ):
@@ -345,7 +374,7 @@ class CInP():
     ( http_code, rec_values, _ ) = self._request( 'UPDATE', uri, data=values, header_map=header_map, timeout=timeout )
 
     if http_code != 200:
-      logging.warning( 'cinp: unexpected HTTP Code "{0}" for UPDATE'.format( http_code ) )
+      logging.warning( 'cinp: Unexpected HTTP Code "{0}" for UPDATE'.format( http_code ) )
       raise ResponseError( 'Unexpected HTTP Code "{0}" for UPDATE'.format( http_code ) )
 
     if not isinstance( rec_values, dict ):
@@ -362,7 +391,7 @@ class CInP():
     ( http_code, _, _ ) = self._request( 'DELETE', uri, timeout=timeout )
 
     if http_code != 200:
-      logging.warning( 'cinp: unexpected HTTP Code "{0}" for DELETE'.format( http_code ) )
+      logging.warning( 'cinp: Unexpected HTTP Code "{0}" for DELETE'.format( http_code ) )
       raise ResponseError( 'Unexpected HTTP Code "{0}" for DELETE'.format( http_code ) )
 
     return True
@@ -382,7 +411,7 @@ class CInP():
     ( http_code, return_value, _ ) = self._request( 'CALL', uri, data=args, header_map=header_map, timeout=timeout )
 
     if http_code != 200:
-      logging.warning( 'cinp: unexpected HTTP Code "{0}" for CALL'.format( http_code ) )
+      logging.warning( 'cinp: Unexpected HTTP Code "{0}" for CALL'.format( http_code ) )
       raise ResponseError( 'Unexpected HTTP Code "{0}" for CALL'.format( http_code ) )
 
     return return_value
@@ -449,7 +478,7 @@ class CInP():
 
     Otherwise a file is created in target_dir, and the full path is returned.  If the
       filename is not specified by the server, and a random filename is chosen.
-      WARNING: there no checking done to make sure the target file does not allready
+      WARNING: there isn't checking done to make sure the target file does not allready
       exist, there is a possibility it could clober something that allready exists.
       we do make sure the filename fits a regex pattern that prevents it from escaping
       the target_dir.  The "filename" as sent by the server is the "model" of the uri.
@@ -480,7 +509,7 @@ class CInP():
 
     http_code = resp.code
     if http_code != 200:
-      logging.warning( 'cinp: unexpected HTTP Code "{0}" for File Get'.format( http_code ) )
+      logging.warning( 'cinp: Unexpected HTTP Code "{0}" for File Get'.format( http_code ) )
       raise ResponseError( 'Unexpected HTTP Code "{0}" for File Get'.format( http_code ) )
 
     try:
@@ -553,7 +582,7 @@ class CInP():
     ( http_code, data, _ ) = self._request( 'UPLOAD', uri_parser.build( namespace, model ), data=file_reader, header_map=header_map, timeout=timeout )
 
     if http_code != 202:
-      logging.warning( 'cinp: unexpected HTTP Code "{0}" for File Upload'.format( http_code ) )
+      logging.warning( 'cinp: Unexpected HTTP Code "{0}" for File Upload'.format( http_code ) )
       raise ResponseError( 'Unexpected HTTP Code "{0}" for File Upload'.format( http_code ) )
 
     return data[ 'uri' ]
