@@ -1,9 +1,11 @@
-from dateutil import parser as datetimeparser
 import traceback
 import json
+import copy
+import sys
+from dateutil import parser as datetimeparser
 from urllib import parse
 
-from cinp.common import URI
+from cinp.common import URI, doccstring_prep
 from cinp.readers import READER_REGISTRY
 
 __CINP_VERSION__ = '0.9'
@@ -12,9 +14,20 @@ __MULTI_URI_MAX__ = 100
 FIELD_TYPE_LIST = ( 'String', 'Integer', 'Float', 'Boolean', 'DateTime', 'Map', 'Model', 'File' )
 
 
+class Notset:
+  def __str__( self ):
+    return "<NOTSET>"
+
+
+notset = Notset()
+
+
 class InvalidRequest( Exception ):
-  def __init__( self, message=None, data=None):
-    self.data = data or { 'message': message } or 'Unknown'
+  def __init__( self, message=None, data=None ):
+    try:
+      self.data = message.response_data
+    except AttributeError:
+      self.data = data or { 'message': str( message ) }
 
   def asResponse( self ):
     return Response( 400, data=self.data )
@@ -50,54 +63,99 @@ class NotAuthorized( Exception ):
   pass
 
 
-class AnonymouseUser():
+class AnonymousUser():
   @property
-  def isSuperuser( self ):
+  def is_superuser( self ):
     return False
 
   @property
-  def isAnonymouse( self ):
+  def is_anonymous( self ):
     return True
 
 
-def _dictConverter( value ):
-  _fromPythonMap( value )
-  return value
+def checkAuth_false( * args ):
+  return False
 
-MAP_TYPE_CONVERTER = {
-                       'NoneType': lambda a: None,
-                       'str': str,
-                       'int': str,
-                       'float': str,
-                       'bool': lambda a: True if a else False,
-                       'datetime': lambda a: a.isoformat(),
-                       'timedelta': lambda a: a.total_seconds(),
-                       'dict': _dictConverter
-                     }
+
+def checkAuth_true( * args ):
+  return True
+
+
+def _debugDump( location, request, exception ):
+  import os
+  from datetime import datetime
+
+  try:
+    if location == '*CONSOLE*':
+      fp = sys.stdout
+    else:
+      fp = open( os.path.join( location, datetime.utcnow().isoformat() ), 'w' )
+
+    fp.write( '** Request **\n' )
+    fp.write( str( request ) )
+    fp.write( '\n\n** Stack **\n' )
+    traceback.print_exception( None, exception, exception.__traceback__, file=fp )
+
+    if fp is not sys.stdout:
+      fp.close()
+
+  except Exception as e:
+    sys.stderr.write( 'Error "{0}" when writing the debug dump'.format( e ) )
 
 
 def _fromPythonMap_converter( value ):
   try:
     return MAP_TYPE_CONVERTER[ type( value ).__name__ ]( value )
   except KeyError:
-    raise ValueError( 'no converter for type "{0}" in map converter'.format( type( value ).__name__ ) )
+    try:
+      return str( value )
+    except Exception:
+      raise ValueError( 'unable to convert type "{0}" in map converter'.format( type( value ).__name__ ) )
+
+
+def _fromPythonList( value ):
+  for index in range( 0, len( value ) ):
+    value[ index ] = _fromPythonMap_converter( value[ index ] )
+
+  return value
 
 
 def _fromPythonMap( value ):
   for key in value.keys():
-    if isinstance( value[ key ], tuple ):  # convert tuple to list before iterating
-      value[ key ] = list( value[ key ] )
+    if not isinstance( key, str ):
+      new_key = str( _fromPythonMap_converter( key ) )  # some types still convert to non-strings(int, float, etc.), JSON requires string
+      assert new_key != key  # if they are the same, bad things happen
+      value[ new_key ] = value[ key ]
+      del value[ key ]
+      key = new_key
 
     if isinstance( value[ key ], dict ):
-      _fromPythonMap( value[ key ] )
+      value[ key ] = _fromPythonMap( value[ key ] )
 
     elif isinstance( value[ key ], list ):
-      for index in range( 0, len( value[ key ] ) ):
-        value[ key ][ index ] = _fromPythonMap_converter( value[ key ][ index ] )
+      value[ key ] = _fromPythonList( value[ key ] )
+
+    elif isinstance( value[ key ], tuple ):
+      value[ key ] = _fromPythonList( list( value[ key ] ) )
 
     else:
       value[ key ] = _fromPythonMap_converter( value[ key ] )
 
+  return value
+
+
+MAP_TYPE_CONVERTER = {
+                       'NoneType': lambda a: None,
+                       'str': lambda a: a,
+                       'int': lambda a: a,
+                       'float': lambda a: a,
+                       'bool': lambda a: True if a else False,
+                       'datetime': lambda a: a.isoformat(),
+                       'timedelta': lambda a: a.total_seconds(),
+                       'dict': _fromPythonMap,
+                       'list': _fromPythonList
+                     }
+
 
 class Converter():
   def __init__( self, uri ):
@@ -249,7 +307,7 @@ class Converter():
       if not isinstance( python_value, dict ):
         raise ValueError( 'Map must be dict' )
 
-      result = python_value.copy()
+      result = copy.deepcopy( python_value )
       _fromPythonMap( result )
 
       return result
@@ -307,10 +365,10 @@ class Converter():
 
 
 class Paramater():
-  def __init__( self, type, name=None, is_array=False, doc=None, length=None, model=None, model_resolve=None, choice_list=None, default=None, allowed_scheme_list=None ):
+  def __init__( self, type, name=None, is_array=False, doc=None, length=None, model=None, model_resolve=None, choice_list=None, default=notset, allowed_scheme_list=None ):
     super().__init__()
     self.name = name
-    self.doc = doc
+    self.doc = doccstring_prep( doc )
     if type is None:
       self.type = None
 
@@ -344,12 +402,12 @@ class Paramater():
       self.choice_list = choice_list
       self.default = default
 
-  def describe( self ):
+  def describe( self, converter ):
     result = { 'name': self.name, 'type': self.type }
-    if self.doc is not None:
+    if self.doc:
       result[ 'doc' ] = self.doc
 
-    if self.type == 'String':
+    if self.type == 'String' and self.length is not None:
       result[ 'length' ] = self.length
 
     if self.type == 'Model':
@@ -363,8 +421,11 @@ class Paramater():
         result[ 'choices' ] = self.choice_list
       if self.is_array:
         result[ 'is_array' ] = True
-      if self.default is not None:
-        result[ 'default' ] = self.default
+      if self.default is not notset:
+        if callable( self.default ):
+          result[ 'default' ] = '<callable "{0}">'.format( self.default.__name__ )
+        else:
+          result[ 'default' ] = converter.fromPython( self, self.default )
 
     return result
 
@@ -378,8 +439,8 @@ class Field( Paramater ):
     self.mode = mode
     self.required = required
 
-  def describe( self ):
-    result = super().describe()
+  def describe( self, converter ):
+    result = super().describe( converter )
     result[ 'mode' ] = self.mode
     result[ 'required' ] = self.required
 
@@ -393,7 +454,7 @@ class Element():
     super().__init__()
     self.parent = None
     self.name = name
-    self.doc = doc
+    self.doc = doccstring_prep( doc )
 
   @property
   def path( self ):
@@ -405,7 +466,7 @@ class Element():
   def startTransaction( self ):
     raise InvalidRequest( 'No Transaction to start' )
 
-  def describe( self ):
+  def describe( self, converter ):
     raise InvalidRequest( 'Not DESCRIBE able' )
 
   def get( self, converter, transaction, id_list, multi ):
@@ -490,8 +551,10 @@ class Namespace( Element ):
     element.parent = self
     self.element_map[ element.name ] = element
 
-  def describe( self ):
-    data = { 'name': self.name, 'path': self.path, 'api-version': self.version, 'multi-uri-max': __MULTI_URI_MAX__, 'doc': self.doc }
+  def describe( self, converter ):
+    data = { 'name': self.name, 'path': self.path, 'api-version': self.version, 'multi-uri-max': __MULTI_URI_MAX__ }
+    if self.doc:
+      data[ 'doc' ] = self.doc
     namespace_list = []
     model_list = []
     for name in self.element_map:
@@ -565,17 +628,19 @@ class Model( Element ):
     action.parent = self
     self.action_map[ action.name ] = action
 
-  def describe( self ):
-    data = { 'name': self.name, 'path': self.path, 'doc': self.doc }
+  def describe( self, converter ):
+    data = { 'name': self.name, 'path': self.path }
+    if self.doc:
+      data[ 'doc' ] = self.doc
     data[ 'constants' ] = {}
     for name in self.constant_set_map:
       data[ 'constants' ][ name ] = self.constant_set_map[ name ]
-    data[ 'fields' ] = [ item.describe() for item in self.field_map.values() ]
+    data[ 'fields' ] = [ item.describe( converter ) for item in self.field_map.values() ]
     data[ 'actions' ] = [ item.path for item in self.action_map.values() ]
-    data[ 'not-allowed-metods' ] = self.not_allowed_verb_list
+    data[ 'not-allowed-methods' ] = self.not_allowed_verb_list
     data[ 'list-filters' ] = {}
     for name in self.list_filter_map:
-      data[ 'list-filters' ][ name ] = [ item.describe() for item in self.list_filter_map[ name ].values() ]
+      data[ 'list-filters' ][ name ] = [ item.describe( converter ) for item in self.list_filter_map[ name ].values() ]
 
     return Response( 200, data=data, header_map={ 'Verb': 'DESCRIBE', 'Type': 'Model', 'Cache-Control': 'max-age=0' } )
 
@@ -595,11 +660,11 @@ class Model( Element ):
     result = {}
     for field_name in self.field_map:
       try:
-        result[ field_name ] = converter.fromPython( self.field_map[ field_name ], getattr( target_object, field_name ) )  # TODO: disguinsh between the AttributeError oflooking up the field, and any errors pulling the field value might cause
+        result[ field_name ] = converter.fromPython( self.field_map[ field_name ], getattr( target_object, field_name ) )  # TODO: disguinsh between the AttributeError of looking up the field, and any errors pulling the field value might cause
       except ValueError as e:
-        raise ValueError( 'Error with "{0}": "{1}"'.format( field_name, str( e ) ) )
+        raise ValueError( 'Error with "{0}": "{1}"'.format( field_name, e ) )
       except AttributeError:
-        raise ServerError( 'target_object missing field "{0}"'.format( field_name ) )  # yes, internal server error, target_object comes from inside the house
+        raise ServerError( 'target_object("{0}") missing field "{1}"'.format( target_object.__class__.__name__, field_name ) )  # yes, internal server error, target_object comes from inside the house
 
     return result
 
@@ -637,6 +702,7 @@ class Model( Element ):
       position = int( header_map.get( 'POSITION', 0 ) )
     except ValueError:
       raise InvalidRequest( 'Count and Position must be integers if specified' )
+
     filter_values = {}
 
     if filter_name is not None:
@@ -654,7 +720,7 @@ class Model( Element ):
         try:
           filter_values[ paramater_name ] = converter.toPython( paramater, data[ paramater_name ], transaction )
         except ValueError as e:
-          error_map[ paramater_name ] = 'Invalid Value "{0}"'.format( str( e ) )
+          error_map[ paramater_name ] = 'Invalid Value "{0}"'.format( e )
         except KeyError:
           error_map[ paramater_name ] = 'Required Paramater'
 
@@ -667,7 +733,7 @@ class Model( Element ):
       if isinstance( e.args[0], dict ):
         raise InvalidRequest( data=e.args[0] )
       else:
-        raise InvalidRequest( str( e ) )
+        raise InvalidRequest( e )
 
     if not isinstance( result, tuple ) and len( result ) != 3:
       raise ServerError( 'List result is not a valid tuple' )
@@ -687,6 +753,7 @@ class Model( Element ):
     value_map = {}
     update_value_map = {}
     error_map = {}
+
     for field_name in data:  # first make sure the fields are ok to look at
       try:
         field = self.field_map[ field_name ]
@@ -709,13 +776,17 @@ class Model( Element ):
           value_map[ field_name ] = converter.toPython( field, data[ field_name ], transaction )
 
       except ValueError as e:
-        error_map[ field_name ] = 'Invalid Value "{0}"'.format( str( e ) )
+        error_map[ field_name ] = 'Invalid Value "{0}"'.format( e )
 
       except KeyError:
-        if field.required:
+        if field.default is not notset:
+          if callable( field.default ):
+            value_map[ field_name ] = field.default()
+          else:
+            value_map[ field_name ] = field.default
+
+        elif field.required:
           error_map[ field_name ] = 'Required Field'
-        else:
-          value_map[ field_name ] = field.default
 
     if error_map != {}:
       raise InvalidRequest( data=error_map )
@@ -726,7 +797,7 @@ class Model( Element ):
       if isinstance( e.args[0], dict ):
         raise InvalidRequest( data=e.args[0] )
       else:
-        raise InvalidRequest( str( e ) )
+        raise InvalidRequest( e )
 
     if not isinstance( result, tuple ) and len( result ) != 2:
       raise ServerError( 'Create result is not a valid tuple' )
@@ -740,7 +811,7 @@ class Model( Element ):
         if isinstance( e.args[0], dict ):
           raise InvalidRequest( data=e.args[0] )
         else:
-          raise InvalidRequest( str( e ) )
+          raise InvalidRequest( e )
 
       if result is None:
         raise ServerError( 'Newly created object disapeared' )
@@ -757,7 +828,7 @@ class Model( Element ):
       if isinstance( e.args[0], dict ):
         raise InvalidRequest( data=e.args[0] )
       else:
-        raise InvalidRequest( str( e ) )
+        raise InvalidRequest( e )
 
     if result is None:
       raise ObjectNotFound( self.path, object_id )
@@ -788,7 +859,7 @@ class Model( Element ):
       try:
         value_map[ field_name ] = converter.toPython( field, data[ field_name ], transaction )
       except ValueError as e:
-        error_map[ field_name ] = 'Invalid Value "{0}"'.format( str( e ) )
+        error_map[ field_name ] = 'Invalid Value "{0}"'.format( e )
       except KeyError:
         pass
 
@@ -842,15 +913,18 @@ class Action( Element ):
 
     return '{0}({1})'.format( self.parent.path, self.name )
 
-  def describe( self ):
-    return_type = self.return_paramater.describe()
+  def describe( self, converter ):
+    return_type = self.return_paramater.describe( converter )
     del return_type[ 'name' ]
-    data = { 'name': self.name, 'path': self.path, 'doc': self.doc, 'return-type': return_type, 'static': self.static }
-    data[ 'paramaters' ] = [ item.describe() for item in self.paramater_map.values() if item.type != '_USER_' ]
+    data = { 'name': self.name, 'path': self.path, 'return-type': return_type, 'static': self.static }
+    if self.doc:
+      data[ 'doc' ] = self.doc
+    data[ 'paramaters' ] = [ item.describe( converter ) for item in self.paramater_map.values() if item.type != '_USER_' ]
 
     return Response( 200, data=data, header_map={ 'Verb': 'DESCRIBE', 'Type': 'Action', 'Cache-Control': 'max-age=0' } )
 
   def call( self, converter, transaction, id_list, data, user, multi ):
+    #  TODO: deal with data when None, and in other places
     error_map = {}
     value_map = {}
     for paramater_name in self.paramater_map:  # should we be ignorning data?
@@ -862,9 +936,17 @@ class Action( Element ):
         try:
           value_map[ paramater_name ] = converter.toPython( paramater, data[ paramater_name ], transaction )
         except KeyError:
-          value_map[ paramater_name ] = paramater.default
+          if paramater.default is not notset:
+            if callable( paramater.default ):
+              value_map[ paramater_name ] = paramater.default()
+            else:
+              value_map[ paramater_name ] = paramater.default
+
+          else:
+            error_map[ paramater_name ] = 'Required Paramater'
+
         except ValueError as e:
-          error_map[ paramater_name ] = 'Invalid Value "{0}"'.format( str( e ) )
+          error_map[ paramater_name ] = 'Invalid Value "{0}"'.format( e )
 
     if error_map != {}:
       raise InvalidRequest( data=error_map )
@@ -881,7 +963,7 @@ class Action( Element ):
         else:
           result = converter.fromPython( self.return_paramater, self.func( self.parent._get( transaction, id_list[0] ), **value_map ) )
       except ValueError as e:
-        raise InvalidRequest( str( e ) )
+        raise InvalidRequest( e )
 
     else:
       if not self.static:
@@ -890,7 +972,7 @@ class Action( Element ):
       try:
         result = converter.fromPython( self.return_paramater, self.func( **value_map ) )
       except ValueError as e:
-        raise InvalidRequest( 'Invalid Result Value: "{0}"'.format( str( e ) ) )
+        raise InvalidRequest( 'Invalid Result Value: "{0}"'.format( e ) )
 
     return Response( 200, data=result, header_map={ 'Verb': 'CALL', 'Cache-Control': 'no-cache', 'Multi-Object': str( multi ) } )
 
@@ -902,17 +984,22 @@ class Action( Element ):
 
 
 class Server():
-  def __init__( self, root_path, root_version, debug=False, cors_allow_list=None ):
+  def __init__( self, root_path, root_version, debug=False, cors_allow_list=None, debug_dump_location=None ):
     super().__init__()
     self.uri = URI( root_path )
     self.debug = debug
+    self.debug_dump_location = debug_dump_location
     self.root_namespace = Namespace( name=None, version=root_version, root_path=root_path, converter=Converter( self.uri ) )
-    self.root_namespace.checkAuth = lambda user, verb, id_list: True
+    self.root_namespace.checkAuth = checkAuth_true
     self.cors_allow_list = cors_allow_list
     self.path_handlers = {}
 
   def getUser( self, auth_id, auth_token ):
-    raise ValueError( 'getUser not implemented' )
+    if auth_id is not None and auth_token is not None:
+      raise ValueError( 'getUser not implemented' )
+
+    else:
+      return AnonymousUser()
 
   def _validateModel( self, model ):
     for field_name in model.field_map:
@@ -985,10 +1072,13 @@ class Server():
           break
 
     except Exception as e:
+      if self.debug_dump_location is not None:
+        _debugDump( self.debug_dump_location, request, e )
+
       if self.debug:
-        response = Response( 500, data={ 'message': 'Path Handler Exception ({0})"{1}"'.format( type( e ).__name__, str( e ) ), 'trace': traceback.format_exc() } )
+        response = Response( 500, data={ 'message': 'Path Handler Exception ({0})"{1}"'.format( type( e ).__name__, e ), 'trace': traceback.format_exc() } )
       else:
-        response = Response( 500, data={ 'message': 'Path Handler Exception ({0})"{1}"'.format( type( e ).__name__, str( e ) ) } )
+        response = Response( 500, data={ 'message': 'Path Handler Exception ({0})"{1}"'.format( type( e ).__name__, e ) } )
 
     if response is None:
       try:
@@ -1007,14 +1097,17 @@ class Server():
         response = Response( 403, data={ 'message': 'Not Authorized' } )
 
       except Exception as e:
+        if self.debug_dump_location is not None:
+          _debugDump( self.debug_dump_location, request, e )
+
         if self.debug:
-          response = Response( 500, data={ 'message': 'Exception ({0})"{1}"'.format( type( e ).__name__, str( e ) ), 'trace': traceback.format_exc() } )
+          response = Response( 500, data={ 'message': 'Exception ({0})"{1}"'.format( type( e ).__name__, e ), 'trace': traceback.format_exc() } )
         else:
-          response = Response( 500, data={ 'message': 'Exception ({0})"{1}"'.format( type( e ).__name__, str( e ) ) } )
+          response = Response( 500, data={ 'message': 'Exception ({0})"{1}"'.format( type( e ).__name__, e ) } )
 
     else:
       if not isinstance( response, Response ):
-        response = Response( 500, data={ 'message': 'Path Handler Return an Invalid Response: ({0})"{1}"'.format( type( response ).__name__, str( response ) ) } )
+        response = Response( 500, data={ 'message': 'Path Handler Return an Invalid Response: ({0})"{1}"'.format( type( response ).__name__, response ) } )
 
     response.header_map[ 'Cinp-Version' ] = __CINP_VERSION__
     if self.cors_allow_list is not None:
@@ -1025,7 +1118,7 @@ class Server():
 
   def dispatch( self, request ):
     if request.verb not in ( 'GET', 'LIST', 'CALL', 'CREATE', 'UPDATE', 'DELETE', 'DESCRIBE', 'OPTIONS' ):
-      return Response( 400, data={ 'message': 'Invalid Verb (HTTP Method)"{0}"'.format( request.verb ) } )
+      return Response( 400, data={ 'message': 'Invalid Verb (HTTP Method) "{0}"'.format( request.verb ) } )
 
     try:
       ( path, model, action, id_list, multi ) = self.uri.split( request.uri )
@@ -1033,13 +1126,13 @@ class Server():
       return Response( 400, data={ 'message': 'Unable to Parse "{0}"'.format( request.uri ) } )
 
     if id_list is not None and len( id_list ) > __MULTI_URI_MAX__:
-      raise InvalidRequest( 'id_list longer than supported length of "{0}"'.format( __MULTI_URI_MAX__ ) )
+      return Response( 400, data={ 'message': 'id_list longer than supported length of "{0}"'.format( __MULTI_URI_MAX__ ) } )
 
     element = self.root_namespace.getElement( ( path, model, action ) )
     if element is None:
-      return Response( 404, 'path not found "{0}"'.format( request.uri ) )
+      return Response( 404, data={ 'message': 'path not found "{0}"'.format( request.uri ) } )
     if not isinstance( element, Element ):
-      return Response( 500, 'confused, path ("{0}") yeilded non element "{1}"'.format( request.uri, element ) )
+      return Response( 500, data={ 'message': 'confused, path ("{0}") yeilded non element "{1}"'.format( request.uri, element ) } )
 
     if request.verb == 'OPTIONS':  # options never need auth, nor is the Cinp-Version header required, we can take care of it early
       response = element.options()
@@ -1053,25 +1146,28 @@ class Server():
       return Response( 400, data={ 'message': 'Invalid CInP Protocol Version' } )
 
     if ( action is not None ) and ( request.verb not in ( 'CALL', 'DESCRIBE' ) ):
-      raise InvalidRequest( 'Invalid verb "{0}" for request with action'.format( request.verb ) )
+      return Response( 400, data={ 'message': 'Invalid verb "{0}" for request with action'.format( request.verb ) } )
 
     if request.verb in ( 'CALL', ) and not isinstance( element, Action ):
-      raise InvalidRequest( 'Verb "{0}" requires action'.format( request.verb ) )
+      return Response( 400, data={ 'message': 'Verb "{0}" requires action'.format( request.verb ) } )
 
     if ( id_list is not None ) and ( request.verb not in ( 'GET', 'UPDATE', 'DELETE', 'CALL' ) ):
-      raise InvalidRequest( 'Invalid Verb "{0}" for request with id'.format( request.verb ) )
+      return Response( 400, data={ 'message': 'Invalid Verb "{0}" for request with id'.format( request.verb ) } )
 
     if ( request.verb in ( 'GET', 'UPDATE', 'DELETE' ) ) and ( id_list is None ):
-      raise InvalidRequest( 'Verb "{0}" requires id'.format( request.verb ) )
+      return Response( 400, data={ 'message': 'Verb "{0}" requires id'.format( request.verb ) } )
 
     if ( request.data is not None ) and ( request.verb not in ( 'LIST', 'UPDATE', 'CREATE', 'CALL' ) ):
-      raise InvalidRequest( 'Invalid verb "{0}" for request with data'.format( request.verb ) )
+      return Response( 400, data={ 'message': 'Invalid verb "{0}" for request with data'.format( request.verb ) } )
 
-    if ( request.verb in ( 'UPDATE', 'CREATE' ) ) and ( request.data is None ):
-      raise InvalidRequest( 'Verb "{0}" requires data'.format( request.verb ) )
+    if ( request.verb in ( 'UPDATE', 'CREATE', 'LIST', 'CALL' ) ) and ( request.data is None ):
+      if request.verb in ( 'LIST', 'CALL' ):
+        request.data = {}
+      else:
+        return Response( 400, data={ 'message': 'Verb "{0}" requires data'.format( request.verb ) } )
 
     if ( request.verb in ( 'GET', 'LIST', 'UPDATE', 'CREATE', 'DELETE' ) ) and not isinstance( element, Model ):
-      raise InvalidRequest( 'Verb "{0}" requires model'.format( request.verb ) )
+      return Response( 400, data={ 'message': 'Verb "{0}" requires model'.format( request.verb ) } )
 
     if ( isinstance( element, Model ) and ( request.verb in element.not_allowed_verb_list ) ) or ( isinstance( element, Action ) and ( request.verb in element.parent.not_allowed_verb_list ) ):
       raise NotAuthorized()
@@ -1086,30 +1182,35 @@ class Server():
 
     auth_id = request.header_map.get( 'AUTH-ID', None )
     auth_token = request.header_map.get( 'AUTH-TOKEN', None )
-    if auth_id is not None and auth_token is not None:
-      user = self.getUser( auth_id, auth_token )
-      if user is None:
-        return Response( 401, data={ 'message': 'Invalid Session' } )
 
-    else:
-      user = AnonymouseUser()
+    user = self.getUser( auth_id, auth_token )
+    if user is None:
+      return Response( 401, data={ 'message': 'Invalid Session' } )
 
-    if not user.isSuperuser:
+    if not user.is_superuser:
       if not element.checkAuth( user, request.verb, id_list ):
         raise NotAuthorized()
 
-    if request.verb == 'DESCRIBE':
-      return element.describe()
-
-    result = None
     if isinstance( element, Action ):
       transaction = element.parent.transaction_class()
       converter = element.parent.parent.converter
-    else:
+    elif isinstance( element, Model ):
       transaction = element.transaction_class()
       converter = element.parent.converter
+    else:
+      transaction = None  # do not need a transaction anyway
+      converter = element.converter
 
+    if request.verb == 'DESCRIBE':
+      return element.describe( converter )
+
+    result = None
     try:
+      in_transaction = False
+      if request.verb in ( 'CREATE', 'UPDATE', 'DELETE', 'CALL' ):
+        transaction.start()
+        in_transaction = True
+
       if request.verb == 'GET':
         result = element.get( converter, transaction, id_list, multi )
 
@@ -1134,14 +1235,17 @@ class Server():
         result = element.call( converter, transaction, id_list, request.data, user, multi )
 
     except Exception as e:
-      transaction.abort()
+      if in_transaction:
+        transaction.abort()
       raise e
 
     if result is None:
-      transaction.abort()
+      if in_transaction:
+        transaction.abort()
       return Response( 500, 'Confused, verb "{0}"'.format( request.verb ) )
 
-    transaction.commit()
+    if in_transaction:
+      transaction.commit()
     return result
 
   def registerNamespace( self, path, namespace ):
@@ -1189,8 +1293,14 @@ class Request():
         self.data = None
         raise InvalidRequest( 'Error Parsing JSON Request data: "{0}"'.format( e ) )
 
-    def fromXML( self, buff ):
-      pass
+  def fromXML( self, buff ):
+    pass
+
+  def fromBytes( self, buff ):
+    pass
+
+  def __str__( self ):
+    return 'Request:\n  Verb: "{0}"\n  URI: "{1}"\n  Header Map: "{2}"\n  Data: "{3}"'.format( self.verb, self.uri, self.header_map, self.data )
 
 
 class Response():
@@ -1222,3 +1332,6 @@ class Response():
 
   def asBytes( self ):
     return None
+
+  def __str__( self ):
+    return 'Response:\n  Content Type: "{0}"\n  HTTP Code: "{1}"\n  Header Map: "{2}"\n  Data: "{3}"'.format( self.content_type, self.http_code, self.header_map, self.data )
