@@ -1,7 +1,7 @@
 import pytest
 
 from cinp.common import URI
-from cinp.server_common import __CINP_VERSION__, Converter, Paramater, Field, Namespace, Model, Action, Request, Response, Server, InvalidRequest, ServerError, ObjectNotFound
+from cinp.server_common import __CINP_VERSION__, Converter, Paramater, Field, Namespace, Model, Action, Request, Response, Server, InvalidRequest, ServerError, ObjectNotFound, AnonymousUser
 
 # TODO: test CORS header stuff
 
@@ -74,6 +74,9 @@ class TestTransaction():
     else:
       return True
 
+  def start( self ):
+    pass
+
   def commit( self ):
     pass
 
@@ -86,15 +89,18 @@ class testUser():
     self.mode = mode
 
   @property
-  def isSuperuser( self ):
+  def is_superuser( self ):
     return self.mode == 'super'
 
   @property
-  def isAnonymouse( self ):
-    return self.mode == 'anonymouse'
+  def is_anonymous( self ):
+    return self.mode == 'anonymous'
 
 
 def getUser( auth_id, auth_token ):
+  if auth_id is None or auth_token is None:
+    return AnonymousUser()
+
   if auth_id == 'super' and auth_token == 'super':
     return testUser( 'super' )
 
@@ -111,7 +117,7 @@ def getUser( auth_id, auth_token ):
 
 
 def checkAuth( user, verb, id_list ):
-  if user.isAnonymouse:
+  if user.is_anonymous:
     return False
 
   if user.mode == 'good':
@@ -125,7 +131,7 @@ def checkAuth( user, verb, id_list ):
 
 def test_namespace():
   ns = Namespace( name=None, version='0.0', root_path='/api/', converter=None )
-  assert sort_dsc( ns.describe().data ) == { 'name': 'root', 'path': '/api/', 'api-version': '0.0', 'namespaces': [], 'models': [], 'multi-uri-max': 100, 'doc': '' }
+  assert sort_dsc( ns.describe( ns.converter ).data ) == { 'name': 'root', 'path': '/api/', 'api-version': '0.0', 'namespaces': [], 'models': [], 'multi-uri-max': 100 }
 
   with pytest.raises( ValueError ):
     Namespace( name='root', version='0.0', converter=None )
@@ -137,33 +143,33 @@ def test_namespace():
     Namespace( name=None, version='0.0', root_path='/asd', converter=None )
 
   ns2 = Namespace( name='ns2', version='2.0', converter=None )
-  assert sort_dsc( ns.describe().data ) == { 'name': 'root', 'path': '/api/', 'api-version': '0.0', 'namespaces': [], 'models': [], 'multi-uri-max': 100, 'doc': '' }
-  assert sort_dsc( ns2.describe().data ) == { 'name': 'ns2', 'path': None, 'api-version': '2.0', 'namespaces': [], 'models': [], 'multi-uri-max': 100, 'doc': '' }
+  assert sort_dsc( ns.describe( ns.converter ).data ) == { 'name': 'root', 'path': '/api/', 'api-version': '0.0', 'namespaces': [], 'models': [], 'multi-uri-max': 100 }
+  assert sort_dsc( ns2.describe( ns2.converter ).data ) == { 'name': 'ns2', 'path': None, 'api-version': '2.0', 'namespaces': [], 'models': [], 'multi-uri-max': 100 }
   ns.addElement( ns2 )
-  assert sort_dsc( ns.describe().data ) == { 'name': 'root', 'path': '/api/', 'api-version': '0.0', 'namespaces': [ '/api/ns2/' ], 'models': [], 'multi-uri-max': 100, 'doc': '' }
-  assert sort_dsc( ns2.describe().data ) == { 'name': 'ns2', 'path': '/api/ns2/', 'api-version': '2.0', 'namespaces': [], 'models': [], 'multi-uri-max': 100, 'doc': '' }
+  assert sort_dsc( ns.describe( ns.converter ).data ) == { 'name': 'root', 'path': '/api/', 'api-version': '0.0', 'namespaces': [ '/api/ns2/' ], 'models': [], 'multi-uri-max': 100 }
+  assert sort_dsc( ns2.describe( ns2.converter ).data ) == { 'name': 'ns2', 'path': '/api/ns2/', 'api-version': '2.0', 'namespaces': [], 'models': [], 'multi-uri-max': 100 }
 
   ns3 = Namespace( name='ns3', version='3.0', converter=None )
-  assert sort_dsc( ns.describe().data ) == { 'name': 'root', 'path': '/api/', 'api-version': '0.0', 'namespaces': [ '/api/ns2/' ], 'models': [], 'multi-uri-max': 100, 'doc': '' }
-  assert sort_dsc( ns2.describe().data ) == { 'name': 'ns2', 'path': '/api/ns2/', 'api-version': '2.0', 'namespaces': [], 'models': [], 'multi-uri-max': 100, 'doc': '' }
-  assert sort_dsc( ns3.describe().data ) == { 'name': 'ns3', 'path': None, 'api-version': '3.0', 'namespaces': [], 'models': [], 'multi-uri-max': 100, 'doc': '' }
+  assert sort_dsc( ns.describe( ns.converter ).data ) == { 'name': 'root', 'path': '/api/', 'api-version': '0.0', 'namespaces': [ '/api/ns2/' ], 'models': [], 'multi-uri-max': 100 }
+  assert sort_dsc( ns2.describe( ns2.converter ).data ) == { 'name': 'ns2', 'path': '/api/ns2/', 'api-version': '2.0', 'namespaces': [], 'models': [], 'multi-uri-max': 100 }
+  assert sort_dsc( ns3.describe( ns3.converter ).data ) == { 'name': 'ns3', 'path': None, 'api-version': '3.0', 'namespaces': [], 'models': [], 'multi-uri-max': 100 }
   ns.addElement( ns3 )
-  assert sort_dsc( ns.describe().data ) == { 'name': 'root', 'path': '/api/', 'api-version': '0.0', 'namespaces': [ '/api/ns2/', '/api/ns3/' ], 'models': [], 'multi-uri-max': 100, 'doc': '' }
-  assert sort_dsc( ns2.describe().data ) == { 'name': 'ns2', 'path': '/api/ns2/', 'api-version': '2.0', 'namespaces': [], 'models': [], 'multi-uri-max': 100, 'doc': '' }
-  assert sort_dsc( ns3.describe().data ) == { 'name': 'ns3', 'path': '/api/ns3/', 'api-version': '3.0', 'namespaces': [], 'models': [], 'multi-uri-max': 100, 'doc': '' }
+  assert sort_dsc( ns.describe( ns.converter ).data ) == { 'name': 'root', 'path': '/api/', 'api-version': '0.0', 'namespaces': [ '/api/ns2/', '/api/ns3/' ], 'models': [], 'multi-uri-max': 100 }
+  assert sort_dsc( ns2.describe( ns2.converter ).data ) == { 'name': 'ns2', 'path': '/api/ns2/', 'api-version': '2.0', 'namespaces': [], 'models': [], 'multi-uri-max': 100 }
+  assert sort_dsc( ns3.describe( ns3.converter ).data ) == { 'name': 'ns3', 'path': '/api/ns3/', 'api-version': '3.0', 'namespaces': [], 'models': [], 'multi-uri-max': 100 }
 
-  ns2_1 = Namespace( name='ns2_1', version='2.1', converter=None )
-  assert sort_dsc( ns.describe().data ) == { 'name': 'root', 'path': '/api/', 'api-version': '0.0', 'namespaces': [ '/api/ns2/', '/api/ns3/' ], 'models': [], 'multi-uri-max': 100, 'doc': '' }
-  assert sort_dsc( ns2.describe().data ) == { 'name': 'ns2', 'path': '/api/ns2/', 'api-version': '2.0', 'namespaces': [], 'models': [], 'multi-uri-max': 100, 'doc': '' }
-  assert sort_dsc( ns3.describe().data ) == { 'name': 'ns3', 'path': '/api/ns3/', 'api-version': '3.0', 'namespaces': [], 'models': [], 'multi-uri-max': 100, 'doc': '' }
-  assert sort_dsc( ns2_1.describe().data ) == { 'name': 'ns2_1', 'path': None, 'api-version': '2.1', 'namespaces': [], 'models': [], 'multi-uri-max': 100, 'doc': '' }
+  ns2_1 = Namespace( name='ns2_1', version='2.1', converter=None, doc='stuff' )
+  assert sort_dsc( ns.describe( ns.converter ).data ) == { 'name': 'root', 'path': '/api/', 'api-version': '0.0', 'namespaces': [ '/api/ns2/', '/api/ns3/' ], 'models': [], 'multi-uri-max': 100 }
+  assert sort_dsc( ns2.describe( ns2.converter ).data ) == { 'name': 'ns2', 'path': '/api/ns2/', 'api-version': '2.0', 'namespaces': [], 'models': [], 'multi-uri-max': 100 }
+  assert sort_dsc( ns3.describe( ns3.converter ).data ) == { 'name': 'ns3', 'path': '/api/ns3/', 'api-version': '3.0', 'namespaces': [], 'models': [], 'multi-uri-max': 100 }
+  assert sort_dsc( ns2_1.describe( ns2_1.converter ).data ) == { 'name': 'ns2_1', 'path': None, 'api-version': '2.1', 'namespaces': [], 'models': [], 'multi-uri-max': 100, 'doc': 'stuff' }
   ns2.addElement( ns2_1 )
-  assert sort_dsc( ns.describe().data ) == { 'name': 'root', 'path': '/api/', 'api-version': '0.0', 'namespaces': [ '/api/ns2/', '/api/ns3/' ], 'models': [], 'multi-uri-max': 100, 'doc': '' }
-  assert sort_dsc( ns2.describe().data ) == { 'name': 'ns2', 'path': '/api/ns2/', 'api-version': '2.0', 'namespaces': [ '/api/ns2/ns2_1/' ], 'models': [], 'multi-uri-max': 100, 'doc': '' }
-  assert sort_dsc( ns3.describe().data ) == { 'name': 'ns3', 'path': '/api/ns3/', 'api-version': '3.0', 'namespaces': [], 'models': [], 'multi-uri-max': 100, 'doc': '' }
-  assert sort_dsc( ns2_1.describe().data ) == { 'name': 'ns2_1', 'path': '/api/ns2/ns2_1/', 'api-version': '2.1', 'namespaces': [], 'models': [], 'multi-uri-max': 100, 'doc': '' }
+  assert sort_dsc( ns.describe( ns.converter ).data ) == { 'name': 'root', 'path': '/api/', 'api-version': '0.0', 'namespaces': [ '/api/ns2/', '/api/ns3/' ], 'models': [], 'multi-uri-max': 100 }
+  assert sort_dsc( ns2.describe( ns2.converter ).data ) == { 'name': 'ns2', 'path': '/api/ns2/', 'api-version': '2.0', 'namespaces': [ '/api/ns2/ns2_1/' ], 'models': [], 'multi-uri-max': 100 }
+  assert sort_dsc( ns3.describe( ns3.converter ).data ) == { 'name': 'ns3', 'path': '/api/ns3/', 'api-version': '3.0', 'namespaces': [], 'models': [], 'multi-uri-max': 100 }
+  assert sort_dsc( ns2_1.describe( ns2_1.converter ).data ) == { 'name': 'ns2_1', 'path': '/api/ns2/ns2_1/', 'api-version': '2.1', 'namespaces': [], 'models': [], 'multi-uri-max': 100, 'doc': 'stuff' }
 
-  assert ns.describe().header_map == { 'Cache-Control': 'max-age=0', 'Verb': 'DESCRIBE', 'Type': 'Namespace' }
+  assert ns.describe( ns.converter ).header_map == { 'Cache-Control': 'max-age=0', 'Verb': 'DESCRIBE', 'Type': 'Namespace' }
 
   assert ns.options().header_map == { 'Allow': 'OPTIONS, DESCRIBE', 'Cache-Control': 'max-age=0' }
   assert ns.options().data is None
@@ -232,26 +238,26 @@ def test_model():
   ns = Namespace( name=None, version='0.0', root_path='/api/', converter=None )
 
   model1 = Model( name='model1', transaction_class=TestTransaction, field_list=[] )
-  assert sort_dsc( ns.describe().data ) == { 'name': 'root', 'path': '/api/', 'api-version': '0.0', 'namespaces': [], 'models': [], 'multi-uri-max': 100, 'doc': '' }
-  assert sort_dsc( model1.describe().data ) == { 'name': 'model1', 'path': None, 'fields': [], 'doc': '', 'actions': [], 'constants': {}, 'list-filters': {}, 'not-allowed-metods': [] }
+  assert sort_dsc( ns.describe( ns.converter ).data ) == { 'name': 'root', 'path': '/api/', 'api-version': '0.0', 'namespaces': [], 'models': [], 'multi-uri-max': 100 }
+  assert sort_dsc( model1.describe( ns.converter ).data ) == { 'name': 'model1', 'path': None, 'fields': [], 'actions': [], 'constants': {}, 'list-filters': {}, 'not-allowed-methods': [] }
   ns.addElement( model1 )
-  assert sort_dsc( ns.describe().data ) == { 'name': 'root', 'path': '/api/', 'api-version': '0.0', 'namespaces': [], 'models': [ '/api/model1' ], 'multi-uri-max': 100, 'doc': '' }
-  assert sort_dsc( model1.describe().data ) == { 'name': 'model1', 'path': '/api/model1', 'fields': [], 'doc': '', 'actions': [], 'constants': {}, 'list-filters': {}, 'not-allowed-metods': [] }
+  assert sort_dsc( ns.describe( ns.converter ).data ) == { 'name': 'root', 'path': '/api/', 'api-version': '0.0', 'namespaces': [], 'models': [ '/api/model1' ], 'multi-uri-max': 100 }
+  assert sort_dsc( model1.describe( model1.parent.converter ).data ) == { 'name': 'model1', 'path': '/api/model1', 'fields': [], 'actions': [], 'constants': {}, 'list-filters': {}, 'not-allowed-methods': [] }
 
   field_list = []
   field_list.append( Field( name='field1', type='String', length=50 ) )
   field_list.append( Field( name='field2', type='Integer', mode='RO' ) )
 
   model2 = Model( name='model2', transaction_class=TestTransaction, field_list=field_list )
-  assert sort_dsc( ns.describe().data ) == { 'name': 'root', 'path': '/api/', 'api-version': '0.0', 'namespaces': [], 'models': [ '/api/model1' ], 'multi-uri-max': 100, 'doc': '' }
-  assert sort_dsc( model1.describe().data ) == { 'name': 'model1', 'path': '/api/model1', 'fields': [], 'doc': '', 'actions': [], 'constants': {}, 'list-filters': {}, 'not-allowed-metods': [] }
-  assert sort_dsc( model2.describe().data ) == { 'name': 'model2', 'path': None, 'fields': [ { 'type': 'String', 'length': 50, 'name': 'field1', 'mode': 'RW', 'required': True, }, { 'type': 'Integer', 'name': 'field2', 'mode': 'RO', 'required': True } ], 'doc': '', 'actions': [], 'constants': {}, 'list-filters': {}, 'not-allowed-metods': [] }
+  assert sort_dsc( ns.describe( ns.converter ).data ) == { 'name': 'root', 'path': '/api/', 'api-version': '0.0', 'namespaces': [], 'models': [ '/api/model1' ], 'multi-uri-max': 100 }
+  assert sort_dsc( model1.describe( model1.parent.converter ).data ) == { 'name': 'model1', 'path': '/api/model1', 'fields': [], 'actions': [], 'constants': {}, 'list-filters': {}, 'not-allowed-methods': [] }
+  assert sort_dsc( model2.describe( ns.converter ).data ) == { 'name': 'model2', 'path': None, 'fields': [ { 'type': 'String', 'length': 50, 'name': 'field1', 'mode': 'RW', 'required': True, }, { 'type': 'Integer', 'name': 'field2', 'mode': 'RO', 'required': True } ], 'actions': [], 'constants': {}, 'list-filters': {}, 'not-allowed-methods': [] }
   ns.addElement( model2 )
-  assert sort_dsc( ns.describe().data ) == { 'name': 'root', 'path': '/api/', 'api-version': '0.0', 'namespaces': [], 'models': [ '/api/model1', '/api/model2' ], 'multi-uri-max': 100, 'doc': '' }
-  assert sort_dsc( model1.describe().data ) == { 'name': 'model1', 'path': '/api/model1', 'fields': [], 'doc': '', 'actions': [], 'constants': {}, 'list-filters': {}, 'not-allowed-metods': [] }
-  assert sort_dsc( model2.describe().data ) == { 'name': 'model2', 'path': '/api/model2', 'fields': [ { 'type': 'String', 'length': 50, 'name': 'field1', 'mode': 'RW', 'required': True }, { 'type': 'Integer', 'name': 'field2', 'mode': 'RO', 'required': True } ], 'doc': '', 'actions': [], 'constants': {}, 'list-filters': {}, 'not-allowed-metods': [] }
+  assert sort_dsc( ns.describe( ns.converter ).data ) == { 'name': 'root', 'path': '/api/', 'api-version': '0.0', 'namespaces': [], 'models': [ '/api/model1', '/api/model2' ], 'multi-uri-max': 100 }
+  assert sort_dsc( model1.describe( model1.parent.converter ).data ) == { 'name': 'model1', 'path': '/api/model1', 'fields': [], 'actions': [], 'constants': {}, 'list-filters': {}, 'not-allowed-methods': [] }
+  assert sort_dsc( model2.describe( model2.parent.converter ).data ) == { 'name': 'model2', 'path': '/api/model2', 'fields': [ { 'type': 'String', 'length': 50, 'name': 'field1', 'mode': 'RW', 'required': True }, { 'type': 'Integer', 'name': 'field2', 'mode': 'RO', 'required': True } ], 'actions': [], 'constants': {}, 'list-filters': {}, 'not-allowed-methods': [] }
 
-  assert model2.describe().header_map == { 'Cache-Control': 'max-age=0', 'Verb': 'DESCRIBE', 'Type': 'Model' }
+  assert model2.describe( model2.parent.converter ).header_map == { 'Cache-Control': 'max-age=0', 'Verb': 'DESCRIBE', 'Type': 'Model' }
 
   assert model2.options().header_map == { 'Allow': 'OPTIONS, DESCRIBE, GET, LIST, CREATE, UPDATE, DELETE' }
   assert model2.options().data is None
@@ -263,18 +269,18 @@ def test_action():
   ns.addElement( model )
 
   action1 = Action( name='act1', return_paramater=Paramater( type='String' ), func=fake_func )
-  assert sort_dsc( action1.describe().data ) == { 'name': 'act1', 'return-type': { 'type': 'String', 'length': None }, 'paramaters': [], 'static': True, 'path': None, 'doc': '' }
+  assert sort_dsc( action1.describe( ns.converter ).data ) == { 'name': 'act1', 'return-type': { 'type': 'String' }, 'paramaters': [], 'static': True, 'path': None }
   model.addAction( action1 )
-  assert sort_dsc( action1.describe().data ) == { 'name': 'act1', 'return-type': { 'type': 'String', 'length': None }, 'paramaters': [], 'static': True, 'path': '/api/model1(act1)', 'doc': '' }
+  assert sort_dsc( action1.describe( action1.parent.parent.converter ).data ) == { 'name': 'act1', 'return-type': { 'type': 'String' }, 'paramaters': [], 'static': True, 'path': '/api/model1(act1)' }
 
   action2 = Action( name='act2', return_paramater=Paramater( type='Integer' ), static=False, func=fake_func )
-  assert sort_dsc( action2.describe().data ) == { 'name': 'act2', 'return-type': { 'type': 'Integer' }, 'paramaters': [], 'static': False, 'path': None, 'doc': '' }
+  assert sort_dsc( action2.describe( ns.converter ).data ) == { 'name': 'act2', 'return-type': { 'type': 'Integer' }, 'paramaters': [], 'static': False, 'path': None }
 
   action3 = Action( name='act3', return_paramater=Paramater( type='Boolean' ), paramater_list=[ Paramater( name='bob', type='Float' ) ], static=False, func=fake_func )
   model.addAction( action3 )
-  assert sort_dsc( action3.describe().data ) == { 'name': 'act3', 'return-type': { 'type': 'Boolean' }, 'paramaters': [ { 'name': 'bob', 'type': 'Float' } ], 'static': False, 'path': '/api/model1(act3)', 'doc': '' }
+  assert sort_dsc( action3.describe( action3.parent.parent.converter ).data ) == { 'name': 'act3', 'return-type': { 'type': 'Boolean' }, 'paramaters': [ { 'name': 'bob', 'type': 'Float' } ], 'static': False, 'path': '/api/model1(act3)' }
 
-  assert action3.describe().header_map == { 'Cache-Control': 'max-age=0', 'Verb': 'DESCRIBE', 'Type': 'Action' }
+  assert action3.describe( action3.parent.parent.converter ).header_map == { 'Cache-Control': 'max-age=0', 'Verb': 'DESCRIBE', 'Type': 'Action' }
 
   assert action3.options().header_map == { 'Allow': 'OPTIONS, DESCRIBE, CALL' }
   assert action3.options().data is None
@@ -600,6 +606,99 @@ def test_response():
   assert resp.asJSON() is None  # yea, still none, base Responose dosen't asXXXX anything
 
 
+def test_saninity_checks():
+  server = Server( root_path='/api/', root_version='0.0' )
+  ns = Namespace( name='ns', version='0.1', converter=None )
+  model = Model( name='model', field_list=[], transaction_class=TestTransaction )
+  ns.addElement( model )
+  action = Action( name='action', return_paramater=Paramater( type='String' ), func=fake_func )
+  model.addAction( action )
+  server.registerNamespace( '/', ns )
+
+  res = server.dispatch( Request( 'BOB', '/api/', { 'CINP-VERSION': '0.9' } ) )
+  assert res.http_code == 400
+  assert res.data == { 'message': 'Invalid Verb (HTTP Method) "BOB"' }
+
+  res = server.dispatch( Request( 'DESCRIBE', 'invalid', { 'CINP-VERSION': '0.9' } ) )
+  assert res.http_code == 400
+  assert res.data == { 'message': 'Unable to Parse "invalid"' }
+
+  res = server.dispatch( Request( 'DESCRIBE', '/api/ns/model:' + ':'.join( 'id' * 101 ) + ':', { 'CINP-VERSION': '0.9' } ) )
+  assert res.http_code == 400
+  assert res.data == { 'message': 'id_list longer than supported length of "100"' }
+
+  res = server.dispatch( Request( 'DESCRIBE', '/api/nope', { 'CINP-VERSION': '0.9' } ) )
+  assert res.http_code == 404
+  assert res.data == { 'message': 'path not found "/api/nope"' }
+
+  res = server.handle( Request( 'DESCRIBE', '/api/', {} ) )
+  assert res.http_code == 400
+  assert res.data == { 'message': 'Invalid CInP Protocol Version' }
+
+  res = server.handle( Request( 'DESCRIBE', '/api/', { 'Cinp-Version': '0' } ) )
+  assert res.http_code == 400
+  assert res.data == { 'message': 'Invalid CInP Protocol Version' }
+
+  with pytest.raises( ValueError ):  # checkAuth not implemented, for this round of tests we call good
+    server.dispatch( Request( 'DESCRIBE', '/api/ns/model(action)', { 'CINP-VERSION': '0.9' } ) )
+
+  for verb in ( 'GET', 'LIST', 'CREATE', 'UPDATE', 'DELETE' ):
+    res = server.dispatch( Request( verb, '/api/ns/model(action)', { 'CINP-VERSION': '0.9' } ) )
+    assert res.http_code == 400
+    assert res.data == { 'message': 'Invalid verb "{0}" for request with action'.format( verb ) }
+
+  for verb in ( 'GET', 'LIST', 'CREATE', 'UPDATE', 'DELETE' ):
+    res = server.dispatch( Request( verb, '/api/ns/model:id:(action)', { 'CINP-VERSION': '0.9' } ) )
+    assert res.http_code == 400
+    assert res.data == { 'message': 'Invalid verb "{0}" for request with action'.format( verb ) }
+
+  for uri in ( '/api/', '/api/ns/', '/api/ns/model', '/api/ns/model:id:' ):
+    res = server.dispatch( Request( 'CALL', uri, { 'CINP-VERSION': '0.9' } ) )
+    assert res.http_code == 400
+    assert res.data == { 'message': 'Verb "CALL" requires action'.format( verb ) }
+
+  for verb in ( 'LIST', 'CREATE', 'DESCRIBE' ):
+    res = server.dispatch( Request( verb, '/api/ns/model:id:', { 'CINP-VERSION': '0.9' } ) )
+    assert res.http_code == 400
+    assert res.data == { 'message': 'Invalid Verb "{0}" for request with id'.format( verb ) }
+
+  for verb in ( 'GET', 'UPDATE', 'DELETE' ):
+    res = server.dispatch( Request( verb, '/api/ns/model', { 'CINP-VERSION': '0.9' } ) )
+    assert res.http_code == 400
+    assert res.data == { 'message': 'Verb "{0}" requires id'.format( verb ) }
+
+  for verb in ( 'GET', 'DELETE' ):
+    req = Request( verb, '/api/ns/model:d:', { 'CINP-VERSION': '0.9' } )
+    req.data = { 'some': 'data' }
+    res = server.dispatch( req )
+    assert res.http_code == 400
+    assert res.data == { 'message': 'Invalid verb "{0}" for request with data'.format( verb ) }
+
+  for verb in ( 'DESCRIBE', ):
+    req = Request( verb, '/api/ns/model', { 'CINP-VERSION': '0.9' } )
+    req.data = { 'some': 'data' }
+    res = server.dispatch( req )
+    assert res.http_code == 400
+    assert res.data == { 'message': 'Invalid verb "{0}" for request with data'.format( verb ) }
+
+  for verb in ( 'UPDATE', ):
+    res = server.dispatch( Request( verb, '/api/ns/model:id:', { 'CINP-VERSION': '0.9' } ) )
+    assert res.http_code == 400
+    assert res.data == { 'message': 'Verb "{0}" requires data'.format( verb ) }
+
+  for verb in ( 'CREATE', ):
+    res = server.dispatch( Request( verb, '/api/ns/model', { 'CINP-VERSION': '0.9' } ) )
+    assert res.http_code == 400
+    assert res.data == { 'message': 'Verb "{0}" requires data'.format( verb ) }
+
+  for verb in ( 'LIST', 'CREATE' ):  # also 'GET', 'UPDATE', 'DELETE' which also requires an Id which requires a model so already covered
+    req = Request( verb, '/api/ns/', { 'CINP-VERSION': '0.9' } )
+    req.data = { 'some': 'data' }
+    res = server.dispatch( req )
+    assert res.http_code == 400
+    assert res.data == { 'message': 'Verb "{0}" requires model'.format( verb ) }
+
+
 def test_server():
   server = Server( root_path='/api/', root_version='0.0', debug=True )
   ns1 = Namespace( name='ns1', version='0.1', converter=None )
@@ -620,21 +719,9 @@ def test_server():
   assert res.http_code == 200
   assert res.header_map == { 'Allow': 'OPTIONS, DESCRIBE', 'Cache-Control': 'max-age=0', 'Cinp-Version': '0.9' }
 
-  req = Request( 'DESCRIBE', '/api/', {} )
-  res = server.handle( req )
-  assert res.http_code == 400
-  assert res.header_map == { 'Cinp-Version': '0.9' }
-  assert res.data == { 'message': 'Invalid CInP Protocol Version' }
-
-  req = Request( 'DESCRIBE', '/api/', { 'Cinp-Version': '0' } )
-  res = server.handle( req )
-  assert res.http_code == 400
-  assert res.header_map == { 'Cinp-Version': '0.9' }
-  assert res.data == { 'message': 'Invalid CInP Protocol Version' }
-
   path = '/api/'
-  desc_ref = sort_dsc( { 'name': 'root', 'path': '/api/', 'api-version': '0.0', 'namespaces': [ '/api/ns1/', '/api/ns2/' ], 'models': [], 'multi-uri-max': 100, 'doc': '' } )
-  assert sort_dsc( server.root_namespace.getElement( server.uri.split( path ) ).describe().data ) == desc_ref
+  desc_ref = sort_dsc( { 'name': 'root', 'path': '/api/', 'api-version': '0.0', 'namespaces': [ '/api/ns1/', '/api/ns2/' ], 'models': [], 'multi-uri-max': 100 } )
+  assert sort_dsc( server.root_namespace.getElement( server.uri.split( path ) ).describe( ns1.converter ).data ) == desc_ref
   req = Request( 'DESCRIBE', path, { 'CINP-VERSION': __CINP_VERSION__ } )
   res = server.handle( req )
   assert res.http_code == 200
@@ -642,8 +729,8 @@ def test_server():
   assert res.header_map == { 'Type': 'Namespace', 'Verb': 'DESCRIBE', 'Cache-Control': 'max-age=0', 'Cinp-Version': '0.9' }
 
   path = '/api/ns1/'
-  desc_ref = sort_dsc( { 'name': 'ns1', 'path': '/api/ns1/', 'api-version': '0.1', 'namespaces': [], 'models': [ '/api/ns1/model1', '/api/ns1/model2' ], 'multi-uri-max': 100, 'doc': '' } )
-  assert sort_dsc( server.root_namespace.getElement( server.uri.split( path ) ).describe().data ) == desc_ref
+  desc_ref = sort_dsc( { 'name': 'ns1', 'path': '/api/ns1/', 'api-version': '0.1', 'namespaces': [], 'models': [ '/api/ns1/model1', '/api/ns1/model2' ], 'multi-uri-max': 100 } )
+  assert sort_dsc( server.root_namespace.getElement( server.uri.split( path ) ).describe( ns1.converter ).data ) == desc_ref
   req = Request( 'DESCRIBE', path, { 'CINP-VERSION': __CINP_VERSION__ } )
   res = server.handle( req )
   assert res.http_code == 200
@@ -651,8 +738,8 @@ def test_server():
   assert res.header_map == { 'Type': 'Namespace', 'Verb': 'DESCRIBE', 'Cache-Control': 'max-age=0', 'Cinp-Version': '0.9' }
 
   path = '/api/ns1/model1'
-  desc_ref = sort_dsc( { 'name': 'model1', 'path': '/api/ns1/model1', 'fields': [], 'doc': '', 'actions': [], 'constants': {}, 'list-filters': {}, 'not-allowed-metods': [] } )
-  assert sort_dsc( server.root_namespace.getElement( server.uri.split( path ) ).describe().data ) == desc_ref
+  desc_ref = sort_dsc( { 'name': 'model1', 'path': '/api/ns1/model1', 'fields': [], 'actions': [], 'constants': {}, 'list-filters': {}, 'not-allowed-methods': [] } )
+  assert sort_dsc( server.root_namespace.getElement( server.uri.split( path ) ).describe( ns1.converter ).data ) == desc_ref
   req = Request( 'DESCRIBE', path, { 'CINP-VERSION': __CINP_VERSION__ } )
   res = server.handle( req )
   assert res.http_code == 200
